@@ -99,7 +99,6 @@ parse_block = function(code, header, params.src, markdown_mode = out_format('mar
   # for quarto, preserve the actual original params.src and do not remove the engine
   if (!is_quarto() || opts_knit$get('tangle')) params.src = params
   params = xfun::csv_options(params)
-  if (is.null(params$label)) params$label = unnamed_chunk()
 
   # remove indent (and possibly markdown blockquote >) from code
   if (nzchar(spaces <- get_chunk_indent(header))) {
@@ -113,10 +112,17 @@ parse_block = function(code, header, params.src, markdown_mode = out_format('mar
   # merge with possible chunk options written as (YAML or CSV) metadata in
   # chunk, and remove metadata from code body
   parts = partition_chunk(engine, code)
+  dup = intersect(names(params), names(parts$options))
   params = merge_list(params, parts$options)
-  code = parts$code
+  if (is.null(params$label)) params$label = unnamed_chunk()
+  label = params$label
+  if (length(dup)) warning(
+    "Duplicated chunk option(s) ", paste0("'", dup, "'", collapse = ', '),
+    " in both chunk header and pipe comments of the chunk '", label, "'.", call. = FALSE
+  )
 
-  label = params$label; .knitEnv$labels = c(.knitEnv$labels, label)
+  code = parts$code
+  .knitEnv$labels = c(.knitEnv$labels, label)
   if (length(code) || length(params[['file']]) || length(params[['code']])) {
     if (label %in% names(knit_code$get())) {
       if (identical(getOption('knitr.duplicate.label'), 'allow')) {
@@ -137,8 +143,9 @@ parse_block = function(code, header, params.src, markdown_mode = out_format('mar
       deps[deps < 0] = length(.knitEnv$labels) + deps[deps < 0]
       deps = .knitEnv$labels[deps[deps > 0]]
     }
-    for (i in deps)
-      dep_list$set(setNames(list(c(dep_list$get(i), label)), i))
+    if (is.character(deps)) {
+      for (i in deps) dep_list$set(setNames(list(c(dep_list$get(i), label)), i))
+    } else warning2("Invalid chunk option 'dependson' (must be character or numeric)")
   }
 
   # for quarto only
@@ -198,7 +205,7 @@ partition_chunk = function(engine, code) {
   opts = options(xfun.handle_error.loc_fun = get_loc)
   on.exit(options(opts))
   # the code has been moved to the xfun package
-  xfun::divide_chunk(engine, code)
+  xfun::divide_chunk(engine, code, strict = is_quarto())
 }
 
 print_block = function(x) {
@@ -225,18 +232,21 @@ parse_inline = function(input, patterns) {
   loc = cbind(start = numeric(0), end = numeric(0))
   if (group_pattern(inline.code)) loc = str_locate(input, inline.code)[[1]]
   code1 = code2 = character()
+  lines = integer()
   if (nrow(loc)) {
     code = t(str_match(input, inline.code))
     if (NCOL(code) >= 2L) {
       code1 = code[, 1L]
       code2 = apply(code[, -1L, drop = FALSE], 1, paste, collapse = '')
+      nl = gregexpr('\n', input, fixed = TRUE)[[1]]
+      lines = if (length(nl) > 1 || nl > -1) findInterval(loc, nl) else 0L
     }
   }
 
-  structure(
-    list(input = input, location = loc, code = code2, code.src = code1),
-    class = 'inline'
-  )
+  structure(list(
+    input = input, location = loc, code = code2, code.src = code1,
+    lines = matrix(lines, ncol = 2, nrow = nrow(loc))
+  ), class = 'inline')
 }
 
 print_inline = function(x) {
@@ -399,12 +409,13 @@ strip_white = function(x, test_strip = is_blank) {
 parse_chunk = function(x, rc = knit_patterns$get('ref.chunk')) {
   if (length(x) == 0L) return(x)
   x = c(x)  # drop attributes of code (e.g. chunk_opts)
-  if (!group_pattern(rc) || !any(idx <- grepl(rc, x))) return(x)
+  if (!group_pattern(rc) || length(idx <- grep(rc, x)) == 0) return(x)
 
   labels = sub(rc, '\\1', x[idx])
-  code = knit_code$get(labels)
+  code = knit_code$get()
+  i = labels %in% names(code)
+  idx = idx[i]; code = code[labels[i]]
   indent = gsub('^(\\s*).*', '\\1', x[idx])
-  if (length(labels) <= 1L) code = list(code)
   code = mapply(indent_block, code, indent, SIMPLIFY = FALSE, USE.NAMES = FALSE)
 
   x = as.list(x)
@@ -469,9 +480,7 @@ match_chunk_end = function(pattern, line, i, b, lines) {
     if (!any(match_chunk_begin(pattern, lines[i + 1:(k - 1)], '^\\1`*\\\\{')))
       return(FALSE)
   }
-  # TODO: clean up the exceptions here (although perhaps some may never update again)
-  signal = if (getOption('knitr.unbalanced.chunk', FALSE)) warning2 else stop2
-  signal(
+  stop2(
     'The closing fence on line ', i, ' ("', line, '") in ', current_input(),
     ' does not match the opening fence "',
     gsub('\\^(\\s*`+).*', '\\1', pattern), '" on line ', b, '. You are recommended to ',
@@ -479,7 +488,6 @@ match_chunk_end = function(pattern, line, i, b, lines) {
     'the same numbers of backticks and same level of indentation (or blockquote). ',
     'See https://yihui.org/en/2021/10/unbalanced-delimiters/ for more info.'
   )
-  TRUE
 }
 
 #' Get all chunk labels in a document
