@@ -595,10 +595,12 @@ html_screenshot = function(x, options = opts_current$get(), ...) {
     switch(options$dev[1], pdf = '.pdf', jpeg = '.jpeg', '.png')
   } else '.png'
   wargs = options$screenshot.opts %n% list()
-  if (is.null(wargs$vwidth)) wargs$vwidth = options$out.width.px
-  if (is.null(wargs$vheight)) wargs$vheight = options$out.height.px
+  if (is.null(wargs$vwidth) && !grepl("%$", W <- options$out.width.px)) wargs$vwidth = W
+  if (is.null(wargs$vheight) && !grepl("%$", H <- options$out.height.px)) wargs$vheight = H
   if (is.null(wargs$delay)) wargs$delay = if (i1) 0.2 else 1
-  d = tempfile()
+  d = tempfile(tmpdir = getOption(
+    'knitr.html_screenshot.tmpdir', if (file.access('.', 2) == 0) '.' else tempdir()
+  ))
   dir.create(d); on.exit(unlink(d, recursive = TRUE), add = TRUE)
   w = webshot_available()
   webshot = c(options$webshot, names(w)[w])
@@ -609,6 +611,8 @@ html_screenshot = function(x, options = opts_current$get(), ...) {
     if (is_quarto()) "fig-format" else "dev", "' to 'png'). ",
     "See https://github.com/yihui/knitr/issues/2276 for more information."
   )
+  # set quiet opions for webshot2::webshot()
+  local_options(list(webshot.quiet = getOption('webshot.quiet', TRUE)))
   f = in_dir(d, {
     if (i1 || i3) {
       if (i1) {
